"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@hypernetlabs/utils");
const neverthrow_1 = require("neverthrow");
const rxjs_1 = require("rxjs");
class HypernetIFrameProxy extends utils_1.ParentProxy {
    constructor(element, iframeUrl, iframeName) {
        super(element, iframeUrl, iframeName);
        this.element = element;
        this.iframeUrl = iframeUrl;
        this.iframeName = iframeName;
        this.isInControl = false;
        this.coreInitialized = false;
        this.coreRegistriesInitialized = false;
        this.coreGovernanceInitialized = false;
        this.corePaymentsInitialized = false;
        this.payments = {
            paymentsInitialized: (chainId) => {
                return this._createCall("paymentsInitialized", chainId);
            },
            waitPaymentsInitialized: (chainId) => {
                if (this.corePaymentsInitialized === true) {
                    return this._createCall("waitPaymentsInitialized", chainId);
                }
                else {
                    return neverthrow_1.ResultAsync.fromSafePromise(this.waitPaymentsInitializedPromise);
                }
            },
            initializePayments: (chainId) => {
                return this._createCall("initializePayments", chainId);
            },
            getPublicIdentifier: () => {
                return this._createCall("getPublicIdentifier", null);
            },
            getActiveStateChannels: () => {
                return this._createCall("getActiveStateChannels", null);
            },
            createStateChannel: (routerPublicIdentifiers, chainId) => {
                return this._createCall("createStateChannel", {
                    routerPublicIdentifiers,
                    chainId,
                });
            },
            depositFunds: (channelAddress, assetAddress, amount) => {
                return this._createCall("depositFunds", {
                    channelAddress,
                    assetAddress,
                    amount: amount,
                });
            },
            withdrawFunds: (channelAddress, assetAddress, amount, destinationAddress) => {
                return this._createCall("withdrawFunds", {
                    channelAddress,
                    assetAddress,
                    amount: amount,
                    destinationAddress,
                });
            },
            getBalances: () => {
                return this._createCall("getBalances", null);
            },
            getLinks: () => {
                return this._createCall("getLinks", null);
            },
            getActiveLinks: () => {
                return this._createCall("getActiveLinks", null);
            },
            acceptOffer: (paymentId) => {
                return this._createCall("acceptFunds", paymentId);
            },
            pullFunds: (paymentId, amount) => {
                return this._createCall("pullFunds", {
                    paymentId,
                    amount: amount,
                });
            },
            finalizePullPayment: (_paymentId, _finalAmount) => {
                throw new Error("Method not implemented.");
            },
            repairPayments: (paymentIds) => {
                return this._createCall("repairPayments", paymentIds);
            },
            mintTestToken: (amount) => {
                return this._createCall("mintTestToken", amount);
            },
            authorizeGateway: (gatewayUrl) => {
                return this._createCall("authorizeGateway", gatewayUrl);
            },
            deauthorizeGateway: (gatewayUrl) => {
                return this._createCall("deauthorizeGateway", gatewayUrl);
            },
            getAuthorizedGateways: () => {
                return this._createCall("getAuthorizedGateways", null);
            },
            getAuthorizedGatewaysConnectorsStatus: () => {
                return this._createCall("getAuthorizedGatewaysConnectorsStatus", null);
            },
            getGatewayTokenInfo: (gatewayUrls) => {
                return this._createCall("getGatewayTokenInfo", gatewayUrls);
            },
            getGatewayRegistrationInfo: (filter) => {
                return this._createCall("getGatewayRegistrationInfo", filter);
            },
            getGatewayEntryList: () => {
                return this._createCall("getGatewayEntryList", null);
            },
            getTokenInformation: () => {
                return this._createCall("getTokenInformation", null);
            },
            getTokenInformationForChain: (chainId) => {
                return this._createCall("getTokenInformationForChain", chainId);
            },
            getTokenInformationByAddress: (tokenAddress) => {
                return this._createCall("getTokenInformationByAddress", tokenAddress);
            },
            displayGatewayIFrame: (gatewayUrl) => {
                return this.payments
                    .getAuthorizedGatewaysConnectorsStatus()
                    .andThen((gatewaysMap) => {
                    if (gatewaysMap.get(gatewayUrl) == true) {
                        this._displayCoreIFrame();
                        return this._createCall("displayGatewayIFrame", gatewayUrl);
                    }
                    else {
                        alert(`Gateway ${gatewayUrl} is not activated at the moment, try again later`);
                        return (0, neverthrow_1.okAsync)(undefined);
                    }
                });
            },
            closeGatewayIFrame: (gatewayUrl) => {
                this._closeCoreIFrame();
                return this._createCall("closeGatewayIFrame", gatewayUrl);
            },
        };
        this.governance = {
            governanceInitialized: (chainId) => {
                return this._createCall("governanceInitialized", chainId);
            },
            waitGovernanceInitialized: (chainId) => {
                if (this.coreGovernanceInitialized === true) {
                    return this._createCall("waitGovernanceInitialized", chainId);
                }
                else {
                    return neverthrow_1.ResultAsync.fromSafePromise(this.waitGovernanceInitializedPromise);
                }
            },
            initializeGovernance: (chainId) => {
                return this._createCall("initializeGovernance", chainId);
            },
            getProposals: (pageNumber, pageSize) => {
                return this._createCall("getProposals", {
                    pageNumber,
                    pageSize,
                });
            },
            createProposal: (name, symbol, owner, enumerable) => {
                return this._createCall("createProposal", {
                    name,
                    symbol,
                    owner,
                    enumerable,
                });
            },
            delegateVote: (delegateAddress, amount) => {
                return this._createCall("delegateVote", {
                    delegateAddress,
                    amount,
                });
            },
            getProposalDetails: (proposalId) => {
                return this._createCall("getProposalDetails", proposalId);
            },
            getProposalDescription: (descriptionHash) => {
                return this._createCall("getProposalDescription", descriptionHash);
            },
            castVote: (proposalId, support) => {
                return this._createCall("castVote", {
                    proposalId,
                    support,
                });
            },
            getProposalVotesReceipt: (proposalId, voterAddress) => {
                return this._createCall("getProposalVotesReceipt", {
                    proposalId,
                    voterAddress,
                });
            },
            getProposalsCount: () => {
                return this._createCall("getProposalsCount", null);
            },
            getProposalThreshold: () => {
                return this._createCall("getProposalThreshold", null);
            },
            getVotingPower: (account) => {
                return this._createCall("getVotingPower", account);
            },
            getHyperTokenBalance: (account) => {
                return this._createCall("getHyperTokenBalance", account);
            },
            queueProposal: (proposalId) => {
                return this._createCall("queueProposal", proposalId);
            },
            cancelProposal: (proposalId) => {
                return this._createCall("cancelProposal", proposalId);
            },
            executeProposal: (proposalId) => {
                return this._createCall("executeProposal", proposalId);
            },
        };
        this.registries = {
            registriesInitialized: (chainId) => {
                return this._createCall("registriesInitialized", chainId);
            },
            waitRegistriesInitialized: (chainId) => {
                if (this.coreRegistriesInitialized === true) {
                    return this._createCall("waitRegistriesInitialized", chainId);
                }
                else {
                    return neverthrow_1.ResultAsync.fromSafePromise(this.waitRegistriesInitializedPromise);
                }
            },
            initializeRegistries: (chainId) => {
                return this._createCall("initializeRegistries", chainId);
            },
            getRegistries: (pageNumber, pageSize, sortOrder) => {
                return this._createCall("getRegistries", {
                    pageNumber,
                    pageSize,
                    sortOrder,
                });
            },
            getRegistryByName: (registryNames) => {
                return this._createCall("getRegistryByName", registryNames);
            },
            getRegistryByAddress: (registryAddresses) => {
                return this._createCall("getRegistryByAddress", registryAddresses);
            },
            getRegistryEntriesTotalCount: (registryNames) => {
                return this._createCall("getRegistryEntriesTotalCount", registryNames);
            },
            getRegistryEntries: (registryName, pageNumber, pageSize, sortOrder) => {
                return this._createCall("getRegistryEntries", {
                    registryName,
                    pageNumber,
                    pageSize,
                    sortOrder,
                });
            },
            getRegistryEntryDetailByTokenId: (registryName, tokenId) => {
                return this._createCall("getRegistryEntryDetailByTokenId", {
                    registryName,
                    tokenId,
                });
            },
            updateRegistryEntryTokenURI: (registryName, tokenId, registrationData) => {
                return this._createCall("updateRegistryEntryTokenURI", {
                    registryName,
                    tokenId,
                    registrationData,
                });
            },
            updateRegistryEntryLabel: (registryName, tokenId, label) => {
                return this._createCall("updateRegistryEntryLabel", {
                    registryName,
                    tokenId,
                    label,
                });
            },
            getNumberOfRegistries: () => {
                return this._createCall("getNumberOfRegistries", null);
            },
            updateRegistryParams: (registryParams) => {
                return this._createCall("updateRegistryParams", registryParams);
            },
            createRegistryEntry: (registryName, newRegistryEntry) => {
                return this._createCall("createRegistryEntry", {
                    registryName,
                    newRegistryEntry,
                });
            },
            transferRegistryEntry: (registryName, tokenId, transferToAddress) => {
                return this._createCall("transferRegistryEntry", {
                    registryName,
                    tokenId,
                    transferToAddress,
                });
            },
            burnRegistryEntry: (registryName, tokenId) => {
                return this._createCall("burnRegistryEntry", {
                    registryName,
                    tokenId,
                });
            },
            createRegistryByToken: (name, symbol, registrarAddress, enumerable) => {
                return this._createCall("createRegistryByToken", {
                    name,
                    symbol,
                    registrarAddress,
                    enumerable,
                });
            },
            grantRegistrarRole: (registryName, address) => {
                return this._createCall("grantRegistrarRole", {
                    registryName,
                    address,
                });
            },
            revokeRegistrarRole: (registryName, address) => {
                return this._createCall("revokeRegistrarRole", {
                    registryName,
                    address,
                });
            },
            renounceRegistrarRole: (registryName, address) => {
                return this._createCall("renounceRegistrarRole", {
                    registryName,
                    address,
                });
            },
            getRegistryEntryByOwnerAddress: (registryName, ownerAddress, index) => {
                return this._createCall("getRegistryEntryByOwnerAddress", {
                    registryName,
                    ownerAddress,
                    index,
                });
            },
            getRegistryModules: () => {
                return this._createCall("getRegistryModules", null);
            },
            createBatchRegistryEntry: (registryName, newRegistryEntries) => {
                return this._createCall("createBatchRegistryEntry", {
                    registryName,
                    newRegistryEntries,
                });
            },
            getRegistryEntryListByOwnerAddress: (registryName, ownerAddress) => {
                return this._createCall("getRegistryEntryListByOwnerAddress", {
                    registryName,
                    ownerAddress,
                });
            },
            getRegistryEntryListByUsername: (registryName, username) => {
                return this._createCall("getRegistryEntryListByUsername", {
                    registryName,
                    username,
                });
            },
            submitLazyMintSignature: (registryName, tokenId, ownerAddress, registrationData) => {
                return this._createCall("submitLazyMintSignature", {
                    registryName,
                    tokenId,
                    ownerAddress,
                    registrationData,
                });
            },
            retrieveLazyMintingSignatures: () => {
                return this._createCall("retrieveLazyMintingSignatures", null);
            },
            executeLazyMint: (lazyMintingSignature) => {
                return this._createCall("executeLazyMint", lazyMintingSignature);
            },
            revokeLazyMintSignature: (lazyMintingSignature) => {
                return this._createCall("revokeLazyMintSignature", lazyMintingSignature);
            },
        };
        this._handshakePromise = null;
        this.onControlClaimed = new rxjs_1.Subject();
        this.onControlYielded = new rxjs_1.Subject();
        this.onPushPaymentSent = new rxjs_1.Subject();
        this.onPullPaymentSent = new rxjs_1.Subject();
        this.onPushPaymentReceived = new rxjs_1.Subject();
        this.onPullPaymentReceived = new rxjs_1.Subject();
        this.onPushPaymentUpdated = new rxjs_1.Subject();
        this.onPullPaymentUpdated = new rxjs_1.Subject();
        this.onPushPaymentDelayed = new rxjs_1.Subject();
        this.onPullPaymentDelayed = new rxjs_1.Subject();
        this.onPushPaymentCanceled = new rxjs_1.Subject();
        this.onPullPaymentCanceled = new rxjs_1.Subject();
        this.onBalancesChanged = new rxjs_1.Subject();
        this.onCeramicAuthenticationStarted = new rxjs_1.Subject();
        this.onCeramicAuthenticationSucceeded = new rxjs_1.Subject();
        this.onCeramicFailed = new rxjs_1.Subject();
        this.onGatewayAuthorized = new rxjs_1.Subject();
        this.onGatewayDeauthorizationStarted = new rxjs_1.Subject();
        this.onAuthorizedGatewayUpdated = new rxjs_1.Subject();
        this.onAuthorizedGatewayActivationFailed = new rxjs_1.Subject();
        this.onGatewayIFrameDisplayRequested = new rxjs_1.Subject();
        this.onGatewayIFrameCloseRequested = new rxjs_1.Subject();
        this.onCoreIFrameDisplayRequested = new rxjs_1.Subject();
        this.onCoreIFrameCloseRequested = new rxjs_1.Subject();
        this.onInitializationRequired = new rxjs_1.Subject();
        this.onPrivateCredentialsRequested = new rxjs_1.Subject();
        this.onWalletConnectOptionsDisplayRequested = new rxjs_1.Subject();
        this.onStateChannelCreated = new rxjs_1.Subject();
        this.onChainConnected = new rxjs_1.Subject();
        this.onGovernanceChainConnected = new rxjs_1.Subject();
        this.onChainChanged = new rxjs_1.Subject();
        this.onAccountChanged = new rxjs_1.Subject();
        this.onGovernanceChainChanged = new rxjs_1.Subject();
        this.onGovernanceAccountChanged = new rxjs_1.Subject();
        this.onGovernanceSignerUnavailable = new rxjs_1.Subject();
        this.initializePromiseResolve = null;
        this.waitInitializedPromise = new Promise((resolve) => {
            this.initializePromiseResolve = resolve;
        });
        this.registriesInitializePromiseResolve = null;
        this.waitRegistriesInitializedPromise = new Promise((resolve) => {
            this.registriesInitializePromiseResolve = resolve;
        });
        this.governanceInitializePromiseResolve = null;
        this.waitGovernanceInitializedPromise = new Promise((resolve) => {
            this.governanceInitializePromiseResolve = resolve;
        });
        this.paymentsInitializePromiseResolve = null;
        this.waitPaymentsInitializedPromise = new Promise((resolve) => {
            this.paymentsInitializePromiseResolve = resolve;
        });
        // Initialize the promise that we'll use to monitor the core
        // initialization status. The iframe will emit an event "initialized"
        // once the core is initialized, we'll use that to resolve this promise.
        this._handshakePromise = this.handshake.then((child) => {
            // Subscribe to the message streams from the iframe,
            // and convert them back to RXJS Subjects.
            child.on("onControlClaimed", (data) => {
                this.isInControl = true;
                this.onControlClaimed.next(data);
            });
            child.on("onControlYielded", (data) => {
                this.isInControl = false;
                this.onControlYielded.next(data);
            });
            child.on("onPushPaymentSent", (data) => {
                this.onPushPaymentSent.next(data);
            });
            child.on("onPullPaymentSent", (data) => {
                this.onPullPaymentSent.next(data);
            });
            child.on("onPushPaymentReceived", (data) => {
                this.onPushPaymentReceived.next(data);
            });
            child.on("onPullPaymentReceived", (data) => {
                this.onPullPaymentReceived.next(data);
            });
            child.on("onPushPaymentUpdated", (data) => {
                this.onPushPaymentUpdated.next(data);
            });
            child.on("onPullPaymentUpdated", (data) => {
                this.onPullPaymentUpdated.next(data);
            });
            child.on("onPushPaymentDelayed", (data) => {
                this.onPushPaymentDelayed.next(data);
            });
            child.on("onPullPaymentDelayed", (data) => {
                this.onPullPaymentDelayed.next(data);
            });
            child.on("onPushPaymentCanceled", (data) => {
                this.onPushPaymentCanceled.next(data);
            });
            child.on("onPullPaymentCanceled", (data) => {
                this.onPullPaymentCanceled.next(data);
            });
            child.on("onBalancesChanged", (data) => {
                this.onBalancesChanged.next(data);
            });
            child.on("onCeramicAuthenticationStarted", () => {
                this._displayCoreIFrame();
                this.onCeramicAuthenticationStarted.next();
            });
            child.on("onCeramicAuthenticationSucceeded", () => {
                this._closeCoreIFrame();
                this.onCeramicAuthenticationSucceeded.next();
            });
            child.on("onCeramicFailed", () => {
                this.onCeramicFailed.next();
            });
            child.on("onGatewayAuthorized", (data) => {
                this.onGatewayAuthorized.next(data);
            });
            child.on("onGatewayDeauthorizationStarted", (data) => {
                this.onGatewayDeauthorizationStarted.next(data);
            });
            child.on("onAuthorizedGatewayUpdated", (data) => {
                this.onAuthorizedGatewayUpdated.next(data);
            });
            child.on("onAuthorizedGatewayActivationFailed", (data) => {
                this.onAuthorizedGatewayActivationFailed.next(data);
            });
            child.on("onStateChannelCreated", (data) => {
                this.onStateChannelCreated.next(data);
            });
            child.on("onChainConnected", (data) => {
                this.onChainConnected.next(data);
            });
            child.on("onGovernanceChainConnected", (data) => {
                this.onGovernanceChainConnected.next(data);
            });
            child.on("onChainChanged", (data) => {
                this.onChainChanged.next(data);
            });
            child.on("onAccountChanged", (data) => {
                this.onAccountChanged.next(data);
            });
            child.on("onGovernanceChainChanged", (data) => {
                this.onGovernanceChainChanged.next(data);
            });
            child.on("onGovernanceAccountChanged", (data) => {
                this.onGovernanceAccountChanged.next(data);
            });
            child.on("onGovernanceSignerUnavailable", (data) => {
                this.onGovernanceSignerUnavailable.next(data);
            });
            // Setup a listener for the "initialized" event.
            child.on("initialized", (data) => {
                // Resolve waitInitialized
                if (this.initializePromiseResolve != null) {
                    this.initializePromiseResolve();
                }
                // And mark us as initialized
                this.coreInitialized = true;
            });
            // Setup a listener for the "registriesInitialized" event.
            child.on("registriesInitialized", (data) => {
                // Resolve waitRegistriesInitialized
                if (this.registriesInitializePromiseResolve != null) {
                    this.registriesInitializePromiseResolve();
                }
                // And mark us as registries initialized
                this.coreRegistriesInitialized = true;
            });
            // Setup a listener for the "governanceInitialized" event.
            child.on("governanceInitialized", (data) => {
                // Resolve waitGovernanceInitialized
                if (this.governanceInitializePromiseResolve != null) {
                    this.governanceInitializePromiseResolve();
                }
                // And mark us as governance initialized
                this.coreGovernanceInitialized = true;
            });
            // Setup a listener for the "paymentsInitialized" event.
            child.on("paymentsInitialized", (data) => {
                // Resolve waitGovernanceInitialized
                if (this.paymentsInitializePromiseResolve != null) {
                    this.paymentsInitializePromiseResolve();
                }
                // And mark us as payments initialized
                this.corePaymentsInitialized = true;
            });
            child.on("onGatewayIFrameDisplayRequested", (data) => {
                this._displayCoreIFrame();
                this.onGatewayIFrameDisplayRequested.next(data);
            });
            child.on("onGatewayIFrameCloseRequested", (data) => {
                this._closeCoreIFrame();
                this.onGatewayIFrameCloseRequested.next(data);
            });
            child.on("onCoreIFrameDisplayRequested", () => {
                this._displayCoreIFrame();
                this.onCoreIFrameDisplayRequested.next();
            });
            child.on("onCoreIFrameCloseRequested", () => {
                this._closeCoreIFrame();
                this.onCoreIFrameCloseRequested.next();
            });
            child.on("onInitializationRequired", () => {
                this.onInitializationRequired.next();
            });
            child.on("onPrivateCredentialsRequested", () => {
                this.onPrivateCredentialsRequested.next();
            });
            child.on("onWalletConnectOptionsDisplayRequested", () => {
                this.onWalletConnectOptionsDisplayRequested.next();
            });
        });
    }
    finalizePullPayment(_paymentId, _finalAmount) {
        throw new Error("Method not implemented.");
    }
    initialized(chainId) {
        // If the child is not initialized, there is no way the core can be.
        if (this.child == null) {
            return (0, neverthrow_1.okAsync)(false);
        }
        // Return the current known status of coreInitialized. We request this
        // information as soon as the child is up.
        return this._createCall("initialized", chainId);
    }
    waitInitialized(chainId) {
        if (this.coreInitialized === true) {
            return this._createCall("waitInitialized", chainId);
        }
        else {
            return neverthrow_1.ResultAsync.fromSafePromise(this.waitInitializedPromise);
        }
    }
    inControl() {
        // If the child is not initialized, there is no way the core can be.
        if (this.child == null) {
            return (0, neverthrow_1.ok)(false);
        }
        // Return the current known status of inControl.
        return (0, neverthrow_1.ok)(this.isInControl);
    }
    getEthereumAccounts() {
        return this._createCall("getEthereumAccounts", null);
    }
    initialize(chainId) {
        return this._createCall("initialize", chainId);
    }
    getInitializationStatus() {
        return this._createCall("getInitializationStatus", null);
    }
    providePrivateCredentials(privateKey, mnemonic) {
        return this._createCall("providePrivateCredentials", {
            privateKey,
            mnemonic,
        });
    }
    getBlockNumber() {
        return this._createCall("getBlockNumber", null);
    }
    provideProviderId(providerId) {
        return this._createCall("provideProviderId", providerId);
    }
    rejectProviderIdRequest() {
        return this._createCall("rejectProviderIdRequest", null);
    }
    retrieveChainInformationList() {
        return this._createCall("retrieveChainInformationList", null);
    }
    retrieveGovernanceChainInformation() {
        return this._createCall("retrieveGovernanceChainInformation", null);
    }
    initializeForChainId(chainId) {
        return this._createCall("initializeForChainId", chainId);
    }
    switchProviderNetwork(chainId) {
        return this._createCall("switchProviderNetwork", chainId);
    }
    getMainProviderChainId() {
        return this._createCall("getMainProviderChainId", null);
    }
    _displayCoreIFrame() {
        // Show core iframe
        if (this.child != null) {
            this.child.frame.style.display = "block";
        }
        // Show core iframe container
        if (this.element != null) {
            this.element.style.display = "block";
        }
    }
    _closeCoreIFrame() {
        // Hide core iframe
        if (this.child != null) {
            this.child.frame.style.display = "none";
        }
        // Hide core iframe container
        if (this.element != null) {
            this.element.style.display = "none";
        }
    }
}
exports.default = HypernetIFrameProxy;
//# sourceMappingURL=HypernetIFrameProxy.js.map